-- get persistent variables
wesnoth.fire("get_global_variable", { namespace="MusicPersist", from_global="MusicPersist", to_local="MusicPersist", side=wesnoth.get_viewing_side() })

-- initialize WMP and session variables
WMP = {}
WMP.option_icon = "ball-green.png"
WMP.portrait_left = "portraits/elves/shyde.png"
WMP.portrait_right = "portraits/elves/sylph.png"
WMP.songs_item_count = 0
WMP.is_playlists = true
WMP.music_selected_index = 1
WMP.is_umc = true

------ 
-- Functions
------
function WMP.log(message)
  wesnoth.log("error", tostring(message), true)
end

function WMP.log_table(variable, indent_level)
  local str = ""
  
  if indent_level == nil then
    indent_level = 0
    str = str.."\n{\n"
  end
  
  local indent = ""
  for i=0,indent_level do
    indent = indent.."    "
  end
  
  for k,v in pairs(variable) do
    if type(v) == "table" then
      str = str..indent.."{\n"
      str = str..WMP.log_table(v, indent_level+1)
      str = str..indent.."},\n"
    else
      str = str..indent
      if type(k) ~= "number" then
        str = str..tostring(k).." = "
      end
      
      if type(v) ~= "string" then
        str = str..tostring(v)..",\n"
      else
        str = str.."\""..v.."\",\n"
      end
    end
  end
  
  if indent_level == 0 then
    str = str.."}"
    wesnoth.log("error", str, false)
  end
  
  return str
end

function WMP.important_text(text)
  return "<span color='#00FF00'>"..text.."</span>"
end

function WMP.strip_path_and_ext(path)
  local slash = string.find(path, "/[^/]*")
  if slash == nil then
    slash = 0
  end
  
  local file_name = string.sub(path, slash+1)
  
  return string.sub(file_name, 1, string.len(file_name)-4)
end

function WMP.execute_playlist(index)
  wesnoth.music_list.clear()
  
  local playlist = WMP.playlists_info[index]
  
  for i=1,#playlist.songs do
    wesnoth.music_list.add(playlist.name.."/"..playlist.songs[i], true)
    wesnoth.music_list[#wesnoth.music_list].shuffle = true
  end
  
  wml.variables["MusicPersist.current_playlist"] = playlist.name
  
  wesnoth.music_list.next()
end

function WMP.write_persist()
  wesnoth.fire("set_global_variable", { namespace="MusicPersist", from_local="MusicPersist", to_global="MusicPersist", side=wesnoth.get_viewing_side(), immediate=true })
end

------
-- Static data tables
------
WMP.umc = wesnoth.require("~add-ons/Wesnoth_Music_Player/GUI2/data/umc.lua")
WMP.changelogs = wesnoth.require("~add-ons/Wesnoth_Music_Player/GUI2/data/changelogs.lua")
WMP.music_locations = wesnoth.require("~add-ons/Wesnoth_Music_Player/GUI2/data/music_locations.lua")
WMP.playlists_info = wesnoth.require("~add-ons/Wesnoth_Music_Player/GUI2/data/playlists_info.lua")

------
-- Pre/post show functions
------
wesnoth.require("~add-ons/Wesnoth_Music_Player/GUI2/functions/info.lua")
wesnoth.require("~add-ons/Wesnoth_Music_Player/GUI2/functions/open.lua")
wesnoth.require("~add-ons/Wesnoth_Music_Player/GUI2/functions/playlists_and_songs.lua")

------
-- Initial logic
------
if wml.variables["MusicPersist.current_playlist"] ~= nil and wesnoth.have_file("~add-ons/Wesnoth_Music_Player_Extras/music/".."/"..wml.variables["MusicPersist.current_playlist"], false) then
  for i=1,#WMP.playlists_info do
    if WMP.playlists_info[i].name == wml.variables["MusicPersist.current_playlist"] then
      WMP.execute_playlist(i)
    end
  end
end
