function WMP.playlists_preshow()
------
-- Local functions
------
  local function refresh_current_previous()
    wesnoth.set_dialog_value(WMP.important_text(WMP.strip_path_and_ext(wesnoth.music_list.current.__cfg.name)), "current_song_value")
    if #wesnoth.music_list > 0 then
      wesnoth.set_dialog_value(WMP.important_text(WMP.strip_path_and_ext(wesnoth.music_list.previous.__cfg.name)), "previous_song_value")
    else
      wesnoth.set_dialog_value(" ", "previous_song_value")
    end
    
    wesnoth.set_dialog_value("<span color='#00FF00'>"..tostring(wml.variables["MusicPersist.current_playlist"]).."</span>", "current_playlist_name")
  end
  
  local function select_playlist()
    wesnoth.remove_dialog_item(1, #WMP.playlists_info[WMP.music_selected_index].songs, "song_list")
    WMP.music_selected_index = wesnoth.get_dialog_value("source_list")
    
    for i=1,#WMP.playlists_info[WMP.music_selected_index].songs do
      wesnoth.set_dialog_value(WMP.option_icon, "song_list", i, "song_icon")
      wesnoth.set_dialog_value(WMP.strip_path_and_ext(WMP.playlists_info[WMP.music_selected_index].songs[i]), "song_list", i, "song_text")
    end
  end
  
  local function set_playlists_on_ui()
    refresh_current_previous()
    
    for i=1,#WMP.playlists_info do
      wesnoth.set_dialog_value(WMP.option_icon, "source_list", i, "sourcelist_icon")
      wesnoth.set_dialog_value(WMP.playlists_info[i].name, "source_list", i, "sourcelist_text")
    end
    
    wesnoth.set_dialog_callback(select_playlist, "source_list")
    select_playlist()
  end
  
  local function select_songlist()
    wesnoth.remove_dialog_item(1, WMP.songs_item_count, "song_list")
    WMP.music_selected_index = wesnoth.get_dialog_value("source_list")
    
    for i=1,#WMP.music_locations[WMP.music_selected_index].songs do
      wesnoth.set_dialog_value(WMP.option_icon, "song_list", i, "song_icon")
      wesnoth.set_dialog_value(WMP.music_locations[WMP.music_selected_index].songs[i], "song_list", i, "song_text")
    end
  end
  
  local function set_songlists_on_ui()
    refresh_current_previous()
    
    for i=1,#WMP.music_locations do
      wesnoth.set_dialog_value(WMP.option_icon, "source_list", i, "sourcelist_icon")
      wesnoth.set_dialog_value(WMP.music_locations[i].name, "source_list", i, "sourcelist_text")
    end
    
    wesnoth.set_dialog_callback(select_songlist, "source_list")
    select_songlist()
  end

------
-- Initialization logic
------
  WMP.is_playlists = true
  wesnoth.set_dialog_value(WMP.portrait_left, "left_image")
  wesnoth.set_dialog_value(WMP.portrait_right, "right_image")
  
  if wesnoth.music_list.volume ~= 100 then
    wesnoth.set_dialog_value("Unmute", "mute_unmute")
  end
  
  set_playlists_on_ui()

------
-- Button click functions
------
  local function play_playlist()
    if WMP.is_playlists then
      WMP.execute_playlist(WMP.music_selected_index)
      refresh_current_previous()
    end
  end
  
  local function play_song()
    local song_index = wesnoth.get_dialog_value("song_list")
    local path = ""
    if WMP.is_playlists then
      path = WMP.playlists_info[WMP.music_selected_index].name.."/"..WMP.playlists_info[WMP.music_selected_index].songs[song_index]
    else
      path = WMP.music_locations[WMP.music_selected_index].songs[song_index]
    end
    
    wesnoth.music_list.play(path)
    refresh_current_previous()
  end
  
  local function next_song()
    wesnoth.music_list.next()
    refresh_current_previous()
  end
  
  local function previous_song()
    if #wesnoth.music_list > 0 then
      wesnoth.music_list.play(wesnoth.music_list.previous.__cfg.name)
    end
    refresh_current_previous()
  end
  
  local function restart_song()
    wesnoth.music_list.play(wesnoth.music_list.current.__cfg.name)
    refresh_current_previous()
  end
  
  local function mute()
    if wesnoth.music_list.volume == 100 then
      wesnoth.music_list.volume = 0
      wesnoth.sound_volume(0)
      wesnoth.set_dialog_value("Unmute", "mute_unmute")
    else
      wesnoth.music_list.volume = 100
      wesnoth.sound_volume(100)
      wesnoth.set_dialog_value("Mute", "mute_unmute")
    end
    refresh_current_previous()
  end
  
  local function swap_songs_playlists()
    if WMP.is_playlists then
      WMP.is_playlists = false
      wesnoth.remove_dialog_item(1, #WMP.playlists_info, "source_list")
      wesnoth.remove_dialog_item(1, #WMP.playlists_info[WMP.music_selected_index].songs, "song_list")
      
      wesnoth.set_dialog_value("Playlists", "swap_songs_playlists")
      set_songlists_on_ui()
    else
      WMP.is_playlists = true
      wesnoth.remove_dialog_item(1, #WMP.music_locations, "source_list")
      wesnoth.remove_dialog_item(1, #WMP.music_locations[WMP.music_selected_index].songs, "song_list")
      
      wesnoth.set_dialog_value("Single Songs", "swap_songs_playlists")
      set_playlists_on_ui()
    end
  end
  
  wesnoth.set_dialog_callback(play_playlist, "play_playlist")
  wesnoth.set_dialog_callback(play_song, "play_song")
  wesnoth.set_dialog_callback(next_song, "next_song")
  wesnoth.set_dialog_callback(previous_song, "previous_song")
  wesnoth.set_dialog_callback(restart_song, "restart_song")
  wesnoth.set_dialog_callback(refresh_current_previous, "refresh_current_previous")
  wesnoth.set_dialog_callback(mute, "mute_unmute")
  wesnoth.set_dialog_callback(swap_songs_playlists, "swap_songs_playlists")
end

function WMP.playlists_postshow()
  
end
